#ifndef KQEMU_H
#define KQEMU_H

#ifdef _WIN32
#define KQEMU_VERSION 0x010101
#else
#define KQEMU_VERSION 0x010000
#endif

struct kqemu_segment_cache {
    uint32_t selector;
    uint32_t base;
    uint32_t limit;
    uint32_t flags;
};

struct kqemu_cpu_state {
    uint32_t regs[8];
    uint32_t eip;
    uint32_t eflags;

    uint32_t dummy0, dummy1, dumm2, dummy3, dummy4;

    struct kqemu_segment_cache segs[6]; /* selector values */
    struct kqemu_segment_cache ldt;
    struct kqemu_segment_cache tr;
    struct kqemu_segment_cache gdt; /* only base and limit are used */
    struct kqemu_segment_cache idt; /* only base and limit are used */

    uint32_t cr0;
    uint32_t dumm5;
    uint32_t cr2;
    uint32_t cr3;
    uint32_t cr4;
    uint32_t a20_mask;

    uint32_t dr0;
    uint32_t dr1;
    uint32_t dr2;
    uint32_t dr3;
    uint32_t dr6;
    uint32_t dr7;

    int cpl; /* currently only 3 */

    uint32_t error_code; /* error_code when exiting with an exception */
    uint32_t next_eip; /* next eip value when exiting with an interrupt */
    unsigned int nb_pages_to_flush; /* number of pages to flush,
                                       KQEMU_FLUSH_ALL means full flush */
#define KQEMU_MAX_PAGES_TO_FLUSH 512
#define KQEMU_FLUSH_ALL (KQEMU_MAX_PAGES_TO_FLUSH + 1)
#ifdef _WIN32
    uint32_t retval;
#endif
};

struct kqemu_init {
    uint8_t *ram_base; /* must be page aligned */
    unsigned long ram_size; /* must be multiple of 4 KB */
    uint8_t *ram_dirty; /* must be page aligned */
    uint32_t **phys_to_ram_map; /* must be page aligned */
    unsigned long *pages_to_flush; /* must be page aligned */
};

#define KQEMU_RET_ABORT    (-1)
#define KQEMU_RET_EXCEPTION 0x0000 /* 8 low order bit are the exception */
#define KQEMU_RET_INT       0x0100 /* 8 low order bit are the interrupt */
#define KQEMU_RET_SOFTMMU   0x0200 /* emulation needed (I/O or
                                      unsupported INSN) */
#define KQEMU_RET_INTR      0x0201 /* interrupted by a signal */

#ifdef _WIN32
#define KQEMU_EXEC           CTL_CODE(FILE_DEVICE_UNKNOWN, 1, METHOD_BUFFERED, FILE_READ_ACCESS | FILE_WRITE_ACCESS)
#define KQEMU_INIT           CTL_CODE(FILE_DEVICE_UNKNOWN, 2, METHOD_BUFFERED, FILE_WRITE_ACCESS)
#define KQEMU_GET_VERSION    CTL_CODE(FILE_DEVICE_UNKNOWN, 3, METHOD_BUFFERED, FILE_READ_ACCESS)
#else
#define KQEMU_EXEC           _IOWR('q', 1, struct kqemu_cpu_state)
#define KQEMU_INIT           _IOW('q', 2, struct kqemu_init)
#define KQEMU_GET_VERSION    _IOR('q', 3, int)
#endif

#endif /* KQEMU_H */
